import requests
import json
from pandas.io.json import json_normalize

from .helpers import generate_dualcase_string
from .discover import Discover
from .helpers import make_discover_request

class DatasetRepo(Discover):
    def __init__(self, endpoint):
        self.endpoint = endpoint

    def query_dataset_by_field(self, field=None, value=None, regexp = False, n = 100):
        if (field == ""):
            payload = { 
                "query": {
                    "multi_match": {
                        "query": value,
                        "fields": []
                    }
                },
            'size' : n
            }

        else:
            payload = { 
            "query": {
                "match": {field: value}
            },
            'size' : n
            }

        if regexp:
            payload  = {'query': {'regexp': 
                    {f'{field}.keyword':
                            {'value': f'.*{generate_dualcase_string(value)}.*',
                            'flags': 'ALL',
                            'max_determinized_states': 10000, 
                            'rewrite': 'constant_score'}}},
                            'size' : n
                            }

        url =  self.host + '/' + self.endpoint + '/_search'
        if field is None or value is None:
            payload = None
        r = make_discover_request('GET', url, json = payload)
        output = r.json()
        search_df = json_normalize(output["hits"]["hits"])
        return search_df

    def query_dataset_by_field_combination(self, and_fields={}, or_fields={},
                                           not_fields={}, n=100):

        and_match_list = [{"match": {k: v}} for k, v in and_fields.items()]
        or_match_list = [{"match": {k: v}} for k, v in or_fields.items()]
        not_match_list = [{"match": {k: v}} for k, v in not_fields.items()]

        payload = {
            "query": {
                "bool": {
                    "must": and_match_list,
                    "must_not": not_match_list,
                    "should": or_match_list
                }
            },
            "size": n
        }
        url = self.host + '/' + self.endpoint + '/_search'
        r = make_discover_request('GET', url, json=payload)
        output = r.json()
        search_df = json_normalize(output["hits"]["hits"])
        return search_df

    def query_dataset_by_field_autocomplete(self, field, value, autofield, regexp = False, n = 10000):
        payload = {
        "_source": ["organism",autofield],
          "query": {
              "match": {field: value}
          },
        'size' : n
        }
        url =  self.host + '/' + self.endpoint + '/_search'
        r = make_discover_request('GET', url, json=payload)
        if r.status_code != 200:
            r = make_discover_request('GET', url, json=payload)
        if r.status_code != 200:
            r = make_discover_request('GET', url, json=payload)
        if r.status_code != 200:
            return r.status_code
        output = r.json()
        search_df = json_normalize(output["hits"]["hits"])
        return search_df
